/* Demonstrate that multiple String references can point to the same place in memory
   and that there is no such thing as reassigning the value of a string because a String is
   immutable, non-changing.

   Concept: A String variable (any object variable) is NOT a container but a pointer to a place in JVM memory. To keep the JVM memory structure stable, and minimize its slowness, it is not possible to make the pointer point to a different place. Instead, memory is conserved by allowing multiple variables (pointers) to point to the same place in memory. In this sense, an object variable is a reference, and not a literal value.

   Analogy: Many newspaper articles can reference the same Declaration of Independence. Only one Declaration of Independence document needs to exist in the Library of Congress.

   Note that the unique ID of a String variable in the JVM is available through the hashCode() method inherited from java.lang.Object. Details at
   http://download.oracle.com/javase/6/docs/api/java/lang/Object.html#hashCode%28%29

   The output is:
     s1's Apples is stored as 1967772793
	 s2's Oranges is stored as 455030757
	 s1's Oranges is stored as 455030757
	 s3's Apples is stored as 1967772793
	 ns1 is stored as 2031692173
	 ns2 is stored as 1660364311
	 ns2 = ns1 is stored as 2031692173
     ns3 = ns2 is stored as 2031692173
*/
class NonString
{
}

class ReferenceTypesIllustratedByComparingsApplesAndOranges
{
	public static void main(String[] args)
	{
		String s1 = "Apples";
		System.out.println("s1's Apples is stored as " + s1.hashCode());
		String s2 = "Oranges";
		System.out.println("s2's Oranges is stored as " + s2.hashCode());

		// reassignment of an existing hashcode to a different value does not exist
		// - instead we reuse a different string's hashcode
		s1 = "Oranges";
		System.out.println("s1's Oranges is stored as " + s1.hashCode());
		String s3 = "Apples";
		System.out.println("s3's Apples is stored as " + s3.hashCode());

		NonString ns1 = new NonString();
		System.out.println("ns1 is stored as " + ns1.hashCode());
		NonString ns2 = new NonString();
		System.out.println("ns2 is stored as " + ns2.hashCode());

		// reassignment of an existing hashcode to a different value does not exist
		// - instead we reuse a different string's hashcode
		ns2 = ns1;
		System.out.println("ns2 = ns1 is stored as " + ns2.hashCode());
		NonString ns3 = ns2;
		System.out.println("ns3 = ns2 is stored as " + ns3.hashCode());
	}
}